#!/usr/bin/python
# -*- coding: euc-kr -*- 
import re
import urllib
from struct import unpack 
from string import lower, strip

def int_o_strip(x):
	if strip(x) == "":
		return 0
	else:
		try:
			return int(strip(x))
		except ValueError:
			return float(strip(x))

def float_o_strip(x):
	if strip(x) == "":
		return 0
	else:
		return float(strip(x))
 
class dbf:	
	def __init__(self, fname):
		self.f = open(fname,'rb') 
		head = self.f.read(32) 
		if (head[0] != '\003') and (head[0] != '\203'): 
			raise TypeError, 'Not a Dbase III+ file!' 
		(self.nrecs, self.hlen, self.rlen) = unpack('4xihh20x', head) 
		fdalen = (self.hlen - 33)/32 

		# read the field descriptor array 
		fda = [] 
		for k in range(fdalen): 
			fda.append(self.f.read(32)) 

		# interpret the field descriptors 
		self.fields = [] 
		for fd in fda: 
			bytes = unpack('12c4xBB14x', fd) 
			field = '' 
			for i in range(11): 
				if bytes[i] == '\000': 
					break 
				field = field+bytes[i] 

			type = bytes[11] 
			length = bytes[12] 
			dec = bytes[13] 
			self.fields.append((field,type,length,dec)) 

		self.hook = {}
		
		for i in range(len(self.fields)):
			field, type, length, dec = self.fields[i]
			pos = i
		
			if type == 'N':
				if dec == 0:
					func = int_o_strip
				else:
					func = float_o_strip
			else:
				func = str

			self.hook[lower(field)] = (pos, func, type, length, dec)
 
	# record numbers go from 0 to self.nrecs-1	
	def _get(self, recno): 
		offs = self.hlen + recno*self.rlen 
		self.f.seek(offs,0) 
		return self.f.read(self.rlen) 
	 
	def __getitem__(self, recno):
		if recno < 0 or recno >= self.nrecs:
			raise IndexError, 'No such record.'
		else:
			raw = self._get(recno) 
			res, pos = [], 0 
			for field in self.fields: 
				end = pos+field[2] 
				item = raw[pos+1:end+1] 
				pos=end 
				res.append(item) 
			# return tuple(res)
			return row(tuple(res), self.hook)

	def status(self):
		print ''
		print 'Header length	 :', self.hlen
		print 'Record length	 :', self.rlen
		print 'Number of records :',  self.nrecs
		print ''
		print '%-12s %-12s %-12s %-12s' % ('Field','Type','Length','Decimal')
		print '%-12s %-12s %-12s %-12s' % ('-----','----','------','-------')
		for k in self.fields:
			print '%-12s %-12s %-12s %-12s' % k
		print ''

class row:
	def __init__(self, ptuple=None, hook=None):
		if ptuple:
			self.data = ptuple
		else:
			raise TypeError, 'pas de row vide !'
		if hook:
			self.hook = hook
		else:
			self.hook = None

	def __repr__(self): 
		return `self.data`

	def __len__(self):
		return len( self.data)

	def __getitem__(self, i):
		return self.data[i]

	def __setitem__(self, i, item):
		raise TypeError, 'immutable row so far'

	def __delitem__(self, i, item):
		raise TypeError, 'immutable row so far'

	def __getattr__(self, aname):
		if not self.hook:
			raise TypeError, 'need a hook'

		if not self.hook.has_key(lower(aname)):
			raise ValueError, ('"%s": no such field !' % aname)
		
		pos, func, type, len, dec = self.hook[lower(aname)]

		return func(self.data[pos])

class postman:
	re_dbf = re.compile('zipcode_[0-9]{8}\([1-4]\).dbf')

	def __init__(self):
		self.prepare()
		self.download()
		self.processdbf()

	def prepare(self):
		content = urllib.urlopen('http://postman.pe.kr/zipcode/index.html').read()
		self.list = self.re_dbf.findall(content)

	def download(self):
		for x in self.list:
			print 'Filename : %s' % x
			print 'Download start...',
			file = 'http://postman.pe.kr/zipcode/%s' % x
			open(x,'w').write(urllib.urlopen(file).read())
			print '   OK'

	def processdbf(self):
		print 'Converting DBF to SQL'
		print 'FileName : %s' % self.list[0]
		db = dbf(self.list[0])
		fp = open(self.list[0]+'.sql','w')
		for r in db:			
			prtstr = 'insert into zipcode ( seq, zipcode, addr1, addr2, addr3, addr4 ) values (\'%s\', \'%s\', \'%s\', \'%s\', \'%s\', \'%s\');' % (r[5].strip(), r[0],r[1],r[2].strip(),r[3].strip(),r[4].strip())
			fp.write(prtstr+'\n')
		db.f.close()
		fp.close()
                print 'Completed'

if __name__ == '__main__':
	d = postman()
